import os
import pickle

def requiredDirectory(path = ''):
	if not os.path.exists(path) or not os.path.isdir(path):
		os.mkdir(path)

class Cache():
	"""docstring for Cache"""

	_suffix = '.cache'
	_defaultCacheDirectory = '__cache__'

	def __init__(self, path = '', prefix = '', name = '', folder = None):
		self._path   = os.path.join(path, self._defaultCacheDirectory)
		self._prefix = prefix
		self._name   = name

		requiredDirectory(self._path)

		if folder:
			self._path = os.path.join(self._path, folder)
			if not os.path.exists(self._path) or not os.path.isdir(self._path):
				os.mkdir(self._path)

	@property
	def path(self):
	    return self._path

	@property
	def name(self):
	    return self._name
	
	@property
	def prefix(self):
	    return self._prefix

	@property
	def fullPath(self):
		return os.path.join(self._path, self._prefix + self._name + self._suffix)

	def save(self, data, attrs = None):
		with open(self.fullPath, 'wb') as f:
			pickle.dump(data, f)
		return True

	def canLoad(self):
		return os.path.exists(self.fullPath)

	def load(self):
		data = None
		if os.path.exists(self.fullPath):
			with open(self.fullPath, 'rb') as f:
				data = pickle.load(f)
		return data