from keras.models import Sequential

from keras.layers.core import Dense, Dropout, Activation, Flatten, Reshape
from keras.layers.normalization import BatchNormalization
from keras.layers.convolutional import Convolution2D, MaxPooling2D
from keras.layers.recurrent import LSTM

from keras.optimizers import Adamax
from keras.optimizers import SGD

class RudyModel(Sequential):

	@staticmethod
	def CertificateToHash(certificate : list) -> str:
		# for c in certificate:
		pass
			

	def __init__(self, input_shape, use_batchnorm = True, dropout = 0.25):
		super(RudyModel, self).__init__()

		self._input_shape  = input_shape
		self.use_batchnorm = use_batchnorm
		self.dropout       = dropout	

	def addConvolutionSequence(self, kernel_count = 32, kernel_width = 2, kernal_height = 2, pool_size = (2,2), activation = 'relu'):

		# If this is the input layer
		if not self.layers:
			self.add(Convolution2D(kernel_count, kernel_width, kernal_height, border_mode = 'same', input_shape = self._input_shape))
		else:
			self.add(Convolution2D(kernel_count, kernel_width, kernal_height, border_mode = 'same'))

		self.add(Activation(activation))
		self.add(Convolution2D(kernel_count, kernel_width, kernal_height))
		self.add(Activation(activation))
		self.add(MaxPooling2D(pool_size = pool_size))
		
		if self.use_batchnorm:
			self.add(BatchNormalization())
		else:
			self.add(Dropout(self.dropout))

	def addFullyConnectedSequence(self, input_dim = 16, activation = 'relu', flatten = True):

		if flatten:
			self.add(Flatten())

		self.add(Dense(input_dim))
		self.add(Activation(activation))
		
		if self.use_batchnorm:
			self.add(BatchNormalization())
		else:
			self.add(Dropout(self.dropout))

	def addLSTMSequence(self, dim = 16, depth = 2, flatten = True):
		if flatten:
			self.add(Flatten())

		for i in range(depth):
			self.add(LSTM(dim, return_sequences = i + 1 != depth))

	def addOutputSequence(self):
		self.add(Dense(1))
		self.add(Activation('sigmoid'))

	def compileAdamax(self, lr = 0.002, beta_1 = 0.9, beta_2 = 0.999, epsilon = 1e-08):
		adamax = Adamax(lr, beta_1, beta_2, epsilon)
		self.compile(
		    loss = 'binary_crossentropy',
		    optimizer = adamax,
		    metrics = ['accuracy'])

	def compileSGD(self, lr = 0.01, decay = 1e-6, momentum = 0.9, nesterov = True):
		sgd = SGD(lr, decay, momentum, nesterov)
		self.compile(
		    loss = 'binary_crossentropy',
		    optimizer = sgd,
		    metrics = ['accuracy'])

	def fitDataset(self, datasets, callbacks, epoch = 100, batch = 64, preprocessor = None):
		X_train = datasets.train.frames.astype('float32')
		Y_train = datasets.train.labels

		# Shuffle test dataset
		datasets.test.doShuffle()

		X_test  = datasets.test.frames.astype('float32')
		Y_test  = datasets.test.labels

		# print('Pre X_Train shape', X_train.shape)
		# print('Pre X_Train Min:', X_train[:,:2].min())
		# print('Pre X_Train Max:', X_train[:,:2].max())

		if preprocessor:
			X_train = preprocessor(X_train)
			X_test  = preprocessor(X_test)

		X_test = X_test[0:1000]
		Y_test = Y_test[0:1000]

		# print('Post X_Train shape', X_train.shape)
		# print('Post X_Train Min:', X_train[:,:2].min())
		# print('Post X_Train Max:', X_train[:,:2].max())

		# Fit the self
		self.fit(X_train, Y_train,
		    batch_size      = batch,
		    nb_epoch        = epoch,
		    validation_data = (X_test, Y_test),
		    shuffle         = True,
		    verbose         = 1,
		    callbacks       = callbacks)

	def saveModel(self, fileName = 'rudy_model.json'):
		json = super().to_json()
		with open(fileName, 'w') as f:
			f.write(json)

	def saveWeights(self, fileName = 'rudy_model_weights.h5'):
		self.save_weights(fileName, overwrite = True)

	def loadWeights(self, fileName = 'rudy_model_weights.h5'):
		self.load_weights(fileName)

	@staticmethod
	def loadFromModel(fileName = 'rudy_model.json'):
		from keras.models import model_from_json
		return model_from_json(fileName)
