import random

from .data_storages import Storage
from .data_sets import DataSets, DataSet, MultiDataSets
from . import utils_logging as log

# Get datasets from local paths, if you pass multiple paths the MultiDataSets will be created. MultiDataSets has same signiture as single DataSets, so you don't have to be worry about usage.
def GetDatasets(paths,
	choosen_side = 'REAR_LEFT',
	batch_names = '1_L13040_112603',
	fixed_beams = [2],
	train_size = 0.7,
	pos_neg_ratio = 1,
	shuffle = False,
	positive_skip = 1,
	negative_skip = 1,
	storage_class = Storage,
	cache_path = '',
	batch_input_shape = (1, 16, 64, 3)):

	log.info('Creating Datasets for %s:' % str(paths))
	log.sep()
	log.info('Choosen Side: %s' % choosen_side)
	log.info('Fixed Beam: %s' % fixed_beams)
	log.info('Train Size: %s' % train_size)
	log.info('Positive / Negative ratio: %s' % pos_neg_ratio)
	log.info('Shuffle is %s' % ('ON' if shuffle else 'OFF'))
	log.info('Batch Input shape: {}'.format(batch_input_shape))
	log.sep()

	islist_batch_names = type(batch_names) in [tuple, list]
	islist_paths = type(paths) in [tuple, list]

	# TODO: Think of better way
	if islist_batch_names:
		# Multiple batch_names
		assert not islist_paths, 'PATHS cannot be list if multiple BATCH_NAMES are defined.'
		ds = []
		for batch_name in batch_names:
			ds.append(CreateDatasets(paths, batch_name, choosen_side, fixed_beams, train_size, pos_neg_ratio, shuffle, positive_skip, negative_skip, storage_class, cache_path, batch_input_shape))
		return MultiDataSets.fromDataSets(ds, name = ds[0].name)
	elif islist_paths:
		# Multiple paths and single batch_names
		ds = []
		for path in paths:
			ds.append(CreateDatasets(paths, batch_names, choosen_side, fixed_beams, train_size, pos_neg_ratio, shuffle, positive_skip, negative_skip, storage_class, cache_path, batch_input_shape))
		return MultiDataSets.fromDataSets(ds, name = ds[0].name)
	else:
		# Single paths and single batch_names
		return CreateDatasets(paths, batch_names, choosen_side, fixed_beams, train_size, pos_neg_ratio, shuffle, positive_skip, negative_skip, storage_class, cache_path, batch_input_shape)

# Create single DataSets from single path
def CreateDatasets(path, batch_name, choosen_side, fixed_beams, train_size, pos_neg_ratio, shuffle, positive_skip, negative_skip, storage_class, cache_path, batch_input_shape):

	# Create Data Sets
	ds = DataSets(
		path 			  = path,
		choosen_side 	  = choosen_side,
		fixed_beams 	  = fixed_beams,
		train_size 		  = train_size,
		use_cache 		  = True,
		cache_path 		  = cache_path,
		batch_name 		  = batch_name,
		batch_input_shape = batch_input_shape)

	if not ds.cached:
		log.warning('Dataset \'{}\' isn\'t cached.')

		storage = storage_class(path, use_cache = use_cache, cache_path = cache_path)

		batch = [batch for batch in storage.batches if batch.name == batch_name]
		if len(batch) != 0:
			batch = batch[0]
		else:
			log.error('No batch for name \'{}\' exists.'.format(batch_name))
			assert False

		if not ds.cached:
			# Set data
			saved = ds.setData(list(batch.framesForSide(choosen_side)), overwrite = True)

			if saved:
				ds.saveCache()

	return ds