import logging

_logger_name = 'rudyLogger'
_logger_setted = False

def getLogger():
	log = logging.getLogger(_logger_name)
	if not _logger_setted:
		setupLogging(log)
	return log

def setupLogging(log = None):
	global _logger_setted
	_logger_setted = True

	log = log or getLogger()
	log.setLevel(logging.DEBUG)

	# Add one default console format
	# addConsoleFormat(log)

def addConsoleFormat(log = None, format = '%(asctime)s: [ %(levelname)s ] %(message)s', level = None):
	'''Add Console handler with specific format and level - output log to console'''
	log = log or getLogger()

	formatter = logging.Formatter(format)
	ch = logging.StreamHandler()
	ch.setFormatter(formatter)
	ch.setLevel(level or logging.INFO)
	log.addHandler(ch)

def addFileFormat(log = None, filename = 'rudy.log', format = '%(asctime)s: [ %(levelname)s ] %(message)s', level = None):
	'''Add File handler with specific format and level - output log to file'''
	log = log or getLogger()

	formatter = logging.Formatter(format)
	ch = logging.FileHandler(filename)
	ch.setFormatter(formatter)
	ch.setLevel(level or logging.INFO)
	log.addHandler(ch)

	# Write Header to file
	from datetime import datetime
	with open(filename, 'a') as l:
		l.write('----------------------------------------------------------------\n')
		l.write('BEGIN LOGGING: %s\n' % datetime.now().strftime('%a, %d %b %Y %X +0000'))
		l.write('----------------------------------------------------------------\n')

def debug(msg):
	log = getLogger()
	log.debug(msg)

def sep():
	log = getLogger()
	log.info('--------------------------------')

def info(msg):
	log = getLogger()
	log.info(msg)

def critical(msg):
	log = getLogger()
	log.critical(msg)

def warning(msg):
	log = getLogger()
	log.warning(msg)

def error(msg):
	log = getLogger()
	log.error(msg)