import time
from collections import defaultdict
import rudy.utils_logging as log

class Stats:
	def __init__(self):
		self.name = ''
		self.time_min = float('inf')
		self.time_max = 0
		self.time_avg = 0
		self.time_duration = 0
		self.count = 0
		self.time_total = 0

	def set(self, duration, name = ''):
		self.name = name
		self.count = self.count + 1
		self.time_min = min(self.time_min, duration)
		self.time_max = max(self.time_max, duration)
		self.time_avg = (self.time_avg + duration) / 2 if self.time_avg != 0 else duration
		self.time_duration = duration
		self.time_total = self.time_total + duration

	@property
	def count_per_sec(self):
		return self.count / self.time_total

	def __str__(self):
		return ''\
			'Name: {name}\n'\
			'Time duration: {duration:.3} sec\n'\
			'Time min/avg/max: {min:.3} / {avg:.3} / {max:.3} sec\n'\
			'Count per sec: {count_per_sec:,} cout/sec\n'.format(
				name = self.name,
				duration = self.time_duration,
				min = self.time_min,
				avg = self.time_avg,
				max = self.time_max,
				count_per_sec = self.count_per_sec)

	def __repr__(self):
		return ''\
			'Name: {name}\n'\
			'Time duration: {duration:.3} sec\n'\
			'Time min/avg/max: {min:.3} / {avg:.3} / {max:.3} sec\n'\
			'Count per sec: {count_per_sec:,} cout/sec\n'.format(
				name = self.name,
				duration = self.time_duration,
				min = self.time_min,
				avg = self.time_avg,
				max = self.time_max,
				count_per_sec = self.count_per_sec)
	
class Measure:
	time_start = defaultdict(float)
	stats = defaultdict(Stats)

	@staticmethod
	def this(name = 'this', print_now = False):
		def this_decorator(func_to_measure):
			def measurable_func(*args, **kwargs):
				Measure.start(name)
				result = func_to_measure(*args, **kwargs)
				stat = Measure.end(name)
				if print_now:
					log.info('{name}:\n{stat!r}'.format(name = name, stat = stat))
				return result
			return measurable_func
		return this_decorator

	@staticmethod
	def clear(name = ''):
		'''Clear the stats for the given Name, if Name is not specified, clear everything.'''
		if not name:
			Measure.time_start.clear()
			Measure.stats.clear()
		else:
			Measure.time_start.pop(name)
			Measure.stats.pop(name)

	@staticmethod
	def start(name):
		Measure.time_start[name] = time.time()

	@staticmethod
	def end(name):
		duration = time.time() - Measure.time_start[name]
		Measure.stats[name].set(duration, name)
		return Measure.stats[name]

	@staticmethod
	def get(name):
		return Measure.stats[name]

	@staticmethod
	def print_stats():
		log.info('[STATS]:')
		for name, stat in Measure.stats.items():
			log.info('Stats for {name}:'.format(name = name))
			log.info(stat)